(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[392],{71350:function(e,t,r){Promise.resolve().then(r.bind(r,19706))},19706:function(e,t,r){"use strict";r.r(t),r.d(t,{default:function(){return u}});var n=r(57437),i=r(43385),o=r(81252),a=r(1801),l=r(23524),s=r(49808),d=r(54662),c=e=>{console.log("renderFilterDropdown",e);let{tempFilteredValue:t,setTempFilteredValue:r,confirm:o,clear:a,close:l}=e;return(0,n.jsx)(i.Qc,{value:t[0],showClear:!0,prefix:(0,n.jsx)(d.Z,{}),placeholder:"搜索... ",onChange:e=>{let t=e?[e]:[];r(t),o({filteredValue:t})},style:{width:200,margin:5}})};function u(){let{Header:e,Footer:t,Sider:r,Content:d}=i.Layout,{data:u,error:h,isLoading:f}=(0,o.ZP)("/v1/streamer-info",a._i);if(f)return(0,n.jsx)(i.yC,{size:"large"});let{Text:m}=i.Typography,g=[{title:"名称",dataIndex:"name",onFilter:(e,t)=>t.name.includes(e),renderFilterDropdown:c},{title:"标题",dataIndex:"title",render:(e,t,r)=>(0,n.jsx)(m,{strong:!0,children:e}),onFilter:(e,t)=>t.title.includes(e),renderFilterDropdown:c},{title:"链接",dataIndex:"url"},{title:"封面",dataIndex:"live_cover_path"},{title:"更新日期",dataIndex:"date",render:(e,t,r)=>(0,n.jsx)(n.Fragment,{children:(0,s.cE)(e)}),sorter:(e,t)=>e.updateTime-t.updateTime>0?1:-1}];return(0,n.jsxs)(n.Fragment,{children:[(0,n.jsx)(e,{style:{backgroundColor:"var(--semi-color-bg-1)"},children:(0,n.jsx)(i.JL,{style:{border:"none"},header:(0,n.jsxs)(n.Fragment,{children:[(0,n.jsx)("div",{style:{backgroundColor:"rgb(250 102 76)",borderRadius:"var(--semi-border-radius-large)",color:"var(--semi-color-bg-0)",display:"flex",padding:"6px"},children:(0,n.jsx)(l.Z,{size:"large"})}),(0,n.jsx)("h4",{style:{marginLeft:"12px"},children:"直播历史"})]}),mode:"horizontal"})}),(0,n.jsx)(d,{style:{paddingLeft:12,paddingRight:12,backgroundColor:"var(--semi-color-bg-0)"},children:(0,n.jsx)("main",{children:(0,n.jsx)(i.iA,{size:"small",rowKey:"id",columns:g,dataSource:u,expandedRowRender:(e,t)=>(0,n.jsxs)(n.Fragment,{children:["文件列表：",e.files.map(e=>(0,n.jsxs)("div",{children:["\xa0\xa0文件名：",e.file]},e.id))]})})})})]})}},1801:function(e,t,r){"use strict";async function n(e,t){let{arg:r}=t;console.log(JSON.stringify(r));let n=await fetch(""+e,{method:"POST",headers:{"Content-Type":"application/json"},body:JSON.stringify(r)});if(!(n.status>=200&&n.status<300))throw Error(await n.text());let i=await n.json();if(!n.ok)throw Error(i.message);return i}r.d(t,{_i:function(){return i},gz:function(){return l},hw:function(){return a},sj:function(){return o},wG:function(){return n}});let i=async function(){for(var e=arguments.length,t=Array(e),r=0;r<e;r++)t[r]=arguments[r];let n=await fetch(""+t[0],t[1]);if(!n.ok)throw Error(await n.text());return n.json()},o=async(e,t)=>{let r=await fetch(""+e,t);if(!r.ok)throw Error(await r.text());return r};async function a(e,t){let{arg:r}=t,n=await fetch("".concat("").concat(e,"/").concat(r),{method:"DELETE"});if(!n.ok)throw Error(await n.text());return n}async function l(e,t){let{arg:r}=t,n=await fetch("".concat("").concat(e),{method:"PUT",headers:{"Content-Type":"application/json"},body:JSON.stringify(r)});if(!n.ok)throw Error(await n.text());return n}},49808:function(e,t,r){"use strict";r.d(t,{cE:function(){return o},f:function(){return n},ql:function(){return i}});let n={xs:"(max-width: 575px)",sm:"(min-width: 576px)",md:"(min-width: 768px)",lg:"(min-width: 992px)",xl:"(min-width: 1200px)",xxl:"(min-width: 1600px)"},i=(e,t)=>{let{match:r,unmatch:n,callInInit:i=!0}=t;{let t=window.matchMedia(e),o=function(e){e.matches?r&&r(e):n&&n(e)};return(i&&o(t),Object.prototype.hasOwnProperty.call(t,"addEventListener"))?(t.addEventListener("change",o),()=>t.removeEventListener("change",o)):(t.addListener(o),()=>t.removeListener(o))}},o=e=>new Date(1e3*e).toLocaleString("zh-CN",{year:"numeric",month:"2-digit",day:"2-digit",hour:"2-digit",minute:"2-digit",second:"2-digit",hour12:!1}).replaceAll("/","-")},23524:function(e,t,r){"use strict";var n=r(2265);let i=(0,r(79238).A)(function(e){return n.createElement("svg",Object.assign({viewBox:"0 0 24 24",fill:"none",xmlns:"http://www.w3.org/2000/svg",width:"1em",height:"1em",focusable:!1,"aria-hidden":!0},e),n.createElement("path",{d:"M6.0824 8.00335C6.5571 5.16399 9.02583 3 12 3C14.9742 3 17.4429 5.16399 17.9176 8.00335C17.4039 8.04525 17 8.47546 17 9V16C17 16.5523 17.4477 17 18 17H18.5322C17.8461 18.3807 16.4717 19.359 14.8579 19.486C14.6831 19.1948 14.3643 19 14 19H10C9.44772 19 9 19.4477 9 20V21C9 21.5523 9.44772 22 10 22H14C14.3744 22 14.7008 21.7942 14.8722 21.4895C17.6318 21.3337 19.9311 19.4565 20.7134 16.9147C22.0259 16.5945 23 15.411 23 14V11C23 9.34315 21.6569 8 20 8H19.9381C19.446 4.05369 16.0796 1 12 1C7.92038 1 4.55399 4.05369 4.06189 8H4C2.34315 8 1 9.34315 1 11V14C1 15.6569 2.34315 17 4 17H6C6.55228 17 7 16.5523 7 16V9C7 8.47546 6.59614 8.04525 6.0824 8.00335Z",fill:"currentColor"}))},"customer_support");t.Z=i}},function(e){e.O(0,[385,252,971,69,744],function(){return e(e.s=71350)}),_N_E=e.O()}]);